@@ -11,17 +11,18 @@
 # - Martin Barisits, <martin.barisits@cern.ch>, 2014
 # - Cedric Serfon, <cedric.serfon@cern.ch>, 2019
 # - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2018
+# - Brandon White, <bjwhite@fnal.gov>, 2019
+# - Eli Chadwick, <eli.chadwick@stfc.ac.uk>, 2020
 #
 # PY3K COMPATIBLE
 
 from sqlalchemy.orm.exc import NoResultFound
-from sqlalchemy.sql.expression import bindparam, text
 
 from rucio.common.exception import CounterNotFound
 import rucio.core.account
 import rucio.core.rse
 
-from rucio.db.sqla import models
+from rucio.db.sqla import models, filter_thread_work
 from rucio.db.sqla.session import read_session, transactional_session
 
 MAX_COUNTERS = 10
@@ -49,8 +50,10 @@ def create_counters_for_new_account(account, session=None):
     :param session: The database session in use.models.RSECounter
     """
 
-    for rse_id in [rse['id'] for rse in rucio.core.rse.list_rses(session=session)]:
-        add_counter(rse_id=rse_id, account=account, session=session)
+    vo = account.vo
+    for rse in rucio.core.rse.list_rses(session=session):
+        if rse['vo'] == vo:
+            add_counter(rse_id=rse['id'], account=account, session=session)
 
 
 @transactional_session
@@ -62,8 +65,10 @@ def create_counters_for_new_rse(rse_id, session=None):
     :param session: The database session in use.models.RSECounter
     """
 
+    vo = rucio.core.rse.get_rse_vo(rse_id, session=session)
     for account in rucio.core.account.list_accounts(session=session):
-        add_counter(rse_id=rse_id, account=account['account'], session=session)
+        if account['account'].vo == vo:
+            add_counter(rse_id=rse_id, account=account['account'], session=session)
 
 
 @transactional_session
@@ -77,7 +82,6 @@ def increase(rse_id, account, files, bytes, session=None):
     :param bytes:   The corresponding amount in bytes.
     :param session: The database session in use.
     """
-
     models.UpdatedAccountCounter(account=account, rse_id=rse_id, files=files, bytes=bytes).save(session=session)
 
 
@@ -92,7 +96,6 @@ def decrease(rse_id, account, files, bytes, session=None):
     :param bytes:   The amount of bytes.
     :param session: The database session in use.
     """
-
     return increase(rse_id=rse_id, account=account, files=-files, bytes=-bytes, session=session)
 
 
@@ -122,15 +125,12 @@ def get_updated_account_counters(total_workers, worker_number, session=None):
     query = session.query(models.UpdatedAccountCounter.account, models.UpdatedAccountCounter.rse_id).\
         distinct(models.UpdatedAccountCounter.account, models.UpdatedAccountCounter.rse_id)
 
-    if total_workers > 0:
-        if session.bind.dialect.name == 'oracle':
-            bindparams = [bindparam('worker_number', worker_number),
-                          bindparam('total_workers', total_workers)]
-            query = query.filter(text('ORA_HASH(CONCAT(account, rse_id), :total_workers) = :worker_number', bindparams=bindparams))
-        elif session.bind.dialect.name == 'mysql':
-            query = query.filter(text('mod(md5(concat(account, rse_id)), %s) = %s' % (total_workers + 1, worker_number)))
-        elif session.bind.dialect.name == 'postgresql':
-            query = query.filter(text('mod(abs((\'x\'||md5(concat(account, rse_id)))::bit(32)::int), %s) = %s' % (total_workers + 1, worker_number)))
+    if session.bind.dialect.name == 'oracle':
+        hash_variable = 'CONCAT(account, rse_id)'''
+    else:
+        hash_variable = 'concat(account, rse_id)'
+
+    query = filter_thread_work(session=session, query=query, total_threads=total_workers, thread_id=worker_number, hash_variable=hash_variable)
 
     return query.all()
 
