@@ -1,4 +1,4 @@
-# Copyright 2013-2018 CERN for the benefit of the ATLAS collaboration.
+# Copyright 2013-2020 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 # - Thomas Beermann, <thomas.beermann@cern.ch>, 2014
 # - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2018
 # - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -109,7 +110,7 @@ def add_subscription(name, account, filter, replication_rules, comments, lifetim
     except IntegrityError as error:
         if re.match('.*IntegrityError.*ORA-00001: unique constraint.*SUBSCRIPTIONS_PK.*violated.*', error.args[0])\
            or re.match(".*IntegrityError.*UNIQUE constraint failed: subscriptions.name, subscriptions.account.*", error.args[0])\
-           or re.match(".*columns name, account are not unique.*", error.args[0])\
+           or re.match('.*IntegrityError.*columns? name.*account.*not unique.*', error.args[0]) \
            or re.match('.*IntegrityError.*ORA-00001: unique constraint.*SUBSCRIPTIONS_NAME_ACCOUNT_UQ.*violated.*', error.args[0])\
            or re.match('.*IntegrityError.*1062.*Duplicate entry.*', error.args[0]) \
            or re.match('.*IntegrityError.*duplicate key value violates unique constraint.*', error.args[0]) \
@@ -202,7 +203,11 @@ def list_subscriptions(name=None, account=None, state=None, session=None):
         if name:
             query = query.filter_by(name=name)
         if account:
-            query = query.filter_by(account=account)
+            if '*' in account.internal:
+                account_str = account.internal.replace('*', '%')
+                query = query.filter(models.Subscription.account.like(account_str))
+            else:
+                query = query.filter_by(account=account)
         if state:
             query = query.filter_by(state=state)
     except IntegrityError as error:
@@ -246,8 +251,14 @@ def list_subscription_rule_states(name=None, account=None, session=None):
     try:
         if name:
             query = query.filter(subscription.name == name)
+
         if account:
-            query = query.filter(subscription.account == account)
+            if '*' in account.internal:
+                account_str = account.internal.replace('*', '%')
+                query = query.filter(subscription.account.like(account_str))
+            else:
+                query = query.filter(subscription.account == account)
+
     except IntegrityError as error:
         print(error)
         raise
