@@ -16,6 +16,8 @@
 # - Martin Barisits <martin.barisits@cern.ch>, 2014-2016
 # - Vincent Garonne <vgaronne@gmail.com>, 2014-2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
+# - Brandon White <bjwhite@fnal.gov>, 2019-2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -56,19 +58,20 @@ def account_update(once=False):
     logging.info('account_update: started')
 
     # Make an initial heartbeat so that all abacus-account daemons have the correct worker number on the next try
+    executable = 'abacus-account'
     hostname = socket.gethostname()
     pid = os.getpid()
     current_thread = threading.current_thread()
-    live(executable='rucio-abacus-account', hostname=hostname, pid=pid, thread=current_thread)
+    live(executable=executable, hostname=hostname, pid=pid, thread=current_thread)
 
     while not graceful_stop.is_set():
         try:
             # Heartbeat
-            heartbeat = live(executable='rucio-abacus-account', hostname=hostname, pid=pid, thread=current_thread)
+            heartbeat = live(executable=executable, hostname=hostname, pid=pid, thread=current_thread)
 
             # Select a bunch of rses for to update for this worker
             start = time.time()  # NOQA
-            account_rse_ids = get_updated_account_counters(total_workers=heartbeat['nr_threads'] - 1,
+            account_rse_ids = get_updated_account_counters(total_workers=heartbeat['nr_threads'],
                                                            worker_number=heartbeat['assign_thread'])
             logging.debug('Index query time %f size=%d' % (time.time() - start, len(account_rse_ids)))
 
@@ -90,7 +93,7 @@ def account_update(once=False):
             break
 
     logging.info('account_update: graceful stop requested')
-    die(executable='rucio-abacus-account', hostname=hostname, pid=pid, thread=current_thread)
+    die(executable=executable, hostname=hostname, pid=pid, thread=current_thread)
     logging.info('account_update: graceful stop done')
 
 
@@ -106,8 +109,9 @@ def run(once=False, threads=1, fill_history_table=False):
     """
     Starts up the Abacus-Account threads.
     """
+    executable = 'abacus-account'
     hostname = socket.gethostname()
-    sanity_check(executable='rucio-abacus-account', hostname=hostname)
+    sanity_check(executable=executable, hostname=hostname)
 
     if once:
         logging.info('main: executing one iteration only')
