@@ -1,4 +1,4 @@
-# Copyright 2013-2018 CERN for the benefit of the ATLAS collaboration.
+# Copyright 2013-2020 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,12 +13,14 @@
 # limitations under the License.
 #
 # Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2015
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2020
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2018
 # - Vincent Garonne <vgaronne@gmail.com>, 2014-2018
 # - Wen Guan <wguan.icedew@gmail.com>, 2014-2016
 # - Martin Barisits <martin.barisits@cern.ch>, 2016-2017
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
+# - Brandon White <bjwhite@fnal.gov>, 2019-2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -66,6 +68,8 @@ graceful_stop = threading.Event()
 
 datetime.datetime.strptime('', '')
 
+TRANSFER_TOOL = config_get('conveyor', 'transfertool', False, None)
+
 
 def poller(once=False, activities=None, sleep_time=60,
            fts_bulk=100, db_bulk=1000, older_than=60, activity_shares=None):
@@ -79,7 +83,7 @@ def poller(once=False, activities=None, sleep_time=60,
     except NoOptionError:
         timeout = None
 
-    executable = sys.argv[0]
+    executable = 'conveyor-poller'
     if activities:
         activities.sort()
         executable += '--activities ' + str(activities)
@@ -92,12 +96,12 @@ def poller(once=False, activities=None, sleep_time=60,
     hb_thread = threading.current_thread()
     heartbeat.sanity_check(executable=executable, hostname=hostname)
     heart_beat = heartbeat.live(executable, hostname, pid, hb_thread)
-    prepend_str = 'Thread [%i/%i] : ' % (heart_beat['assign_thread'] + 1, heart_beat['nr_threads'])
+    prepend_str = 'Thread [%i/%i] : ' % (heart_beat['assign_thread'], heart_beat['nr_threads'])
     logging.info(prepend_str + 'Poller starting - db_bulk (%i) fts_bulk (%i) timeout (%s)' % (db_bulk, fts_bulk, timeout))
 
     time.sleep(10)  # To prevent running on the same partition if all the poller restart at the same time
     heart_beat = heartbeat.live(executable, hostname, pid, hb_thread)
-    prepend_str = 'Thread [%i/%i] : ' % (heart_beat['assign_thread'] + 1, heart_beat['nr_threads'])
+    prepend_str = 'Thread [%i/%i] : ' % (heart_beat['assign_thread'], heart_beat['nr_threads'])
 
     logging.info(prepend_str + 'Poller started')
 
@@ -107,7 +111,7 @@ def poller(once=False, activities=None, sleep_time=60,
 
         try:
             heart_beat = heartbeat.live(executable, hostname, pid, hb_thread, older_than=3600)
-            prepend_str = 'Thread [%i/%i] : ' % (heart_beat['assign_thread'] + 1, heart_beat['nr_threads'])
+            prepend_str = 'Thread [%i/%i] : ' % (heart_beat['assign_thread'], heart_beat['nr_threads'])
 
             if activities is None:
                 activities = [None]
@@ -122,7 +126,7 @@ def poller(once=False, activities=None, sleep_time=60,
                                                 state=[RequestState.SUBMITTED],
                                                 limit=db_bulk,
                                                 older_than=datetime.datetime.utcnow() - datetime.timedelta(seconds=older_than),
-                                                total_workers=heart_beat['nr_threads'] - 1, worker_number=heart_beat['assign_thread'],
+                                                total_workers=heart_beat['nr_threads'], worker_number=heart_beat['assign_thread'],
                                                 mode_all=False, hash_variable='id',
                                                 activity=activity,
                                                 activity_shares=activity_shares)
@@ -230,10 +234,16 @@ def poll_transfers(external_host, xfers, prepend_str='', request_ids=None, timeo
     :param timeout:          Timeout.
     """
     try:
+        if TRANSFER_TOOL == 'mock':
+            logging.debug(prepend_str + 'Setting %s transfer requests status to DONE per mock tool' % (len(xfers)))
+            for task_id in xfers:
+                ret = transfer_core.update_transfer_state(external_host=None, transfer_id=task_id, state=RequestState.DONE)
+                record_counter('daemons.conveyor.poller.update_request_state.%s' % ret)
+            return
         try:
             tss = time.time()
             logging.info(prepend_str + 'Polling %i transfers against %s with timeout %s' % (len(xfers), external_host, timeout))
-            resps = transfer_core.bulk_query_transfers(external_host, xfers, 'fts3', timeout)
+            resps = transfer_core.bulk_query_transfers(external_host, xfers, TRANSFER_TOOL, timeout)
             record_timer('daemons.conveyor.poller.bulk_query_transfers', (time.time() - tss) * 1000 / len(xfers))
         except TransferToolTimeout as error:
             logging.error(prepend_str + str(error))
@@ -244,8 +254,7 @@ def poll_transfers(external_host, xfers, prepend_str='', request_ids=None, timeo
             for xfer in xfers:
                 try:
                     logging.debug(prepend_str + 'Checking %s on %s' % (xfer, external_host))
-                    status = transfer_core.bulk_query_transfers(external_host, [xfer, ], 'fts3', timeout)
-                    logging.debug(prepend_str + str(status))
+                    status = transfer_core.bulk_query_transfers(external_host, [xfer, ], TRANSFER_TOOL, timeout)
                     if xfer in status and isinstance(status[xfer], Exception):
                         logging.error(prepend_str + 'Problem querying %s on %s . Error returned : %s' % (xfer, external_host, str(status[xfer])))
                 except Exception as err:
@@ -263,36 +272,42 @@ def poll_transfers(external_host, xfers, prepend_str='', request_ids=None, timeo
         tss = time.time()
         logging.debug(prepend_str + 'Updating %s transfer requests status' % (len(xfers)))
         cnt = 0
-        for transfer_id in resps:
-            try:
-                transf_resp = resps[transfer_id]
-                # transf_resp is None: Lost.
-                #             is Exception: Failed to get fts job status.
-                #             is {}: No terminated jobs.
-                #             is {request_id: {file_status}}: terminated jobs.
-                if transf_resp is None:
-                    transfer_core.update_transfer_state(external_host, transfer_id, RequestState.LOST, logging_prepend_str=prepend_str)
-                    record_counter('daemons.conveyor.poller.transfer_lost')
-                elif isinstance(transf_resp, Exception):
-                    logging.warning(prepend_str + "Failed to poll FTS(%s) job (%s): %s" % (external_host, transfer_id, transf_resp))
-                    record_counter('daemons.conveyor.poller.query_transfer_exception')
-                else:
-                    for request_id in transf_resp:
-                        if request_id in request_ids:
-                            ret = request_core.update_request_state(transf_resp[request_id], logging_prepend_str=prepend_str)
-                            # if True, really update request content; if False, only touch request
-                            if ret:
-                                cnt += 1
-                            record_counter('daemons.conveyor.poller.update_request_state.%s' % ret)
 
-                # should touch transfers.
-                # Otherwise if one bulk transfer includes many requests and one is not terminated, the transfer will be poll again.
-                transfer_core.touch_transfer(external_host, transfer_id)
-            except (DatabaseException, DatabaseError) as error:
-                if re.match('.*ORA-00054.*', error.args[0]) or re.match('.*ORA-00060.*', error.args[0]) or 'ERROR 1205 (HY000)' in error.args[0]:
-                    logging.warn(prepend_str + "Lock detected when handling request %s - skipping" % request_id)
-                else:
-                    logging.error(traceback.format_exc())
-        logging.debug(prepend_str + 'Finished updating %s transfer requests status (%i requests state changed) in %s seconds' % (len(xfers), cnt, (time.time() - tss)))
+        if TRANSFER_TOOL == 'globus':
+            for task_id in resps:
+                ret = transfer_core.update_transfer_state(external_host=None, transfer_id=task_id, state=resps[task_id])
+                record_counter('daemons.conveyor.poller.update_request_state.%s' % ret)
+        else:
+            for transfer_id in resps:
+                try:
+                    transf_resp = resps[transfer_id]
+                    # transf_resp is None: Lost.
+                    #             is Exception: Failed to get fts job status.
+                    #             is {}: No terminated jobs.
+                    #             is {request_id: {file_status}}: terminated jobs.
+                    if transf_resp is None:
+                        transfer_core.update_transfer_state(external_host, transfer_id, RequestState.LOST, logging_prepend_str=prepend_str)
+                        record_counter('daemons.conveyor.poller.transfer_lost')
+                    elif isinstance(transf_resp, Exception):
+                        logging.warning(prepend_str + "Failed to poll FTS(%s) job (%s): %s" % (external_host, transfer_id, transf_resp))
+                        record_counter('daemons.conveyor.poller.query_transfer_exception')
+                    else:
+                        for request_id in transf_resp:
+                            if request_id in request_ids:
+                                ret = request_core.update_request_state(transf_resp[request_id], logging_prepend_str=prepend_str)
+                                # if True, really update request content; if False, only touch request
+                                if ret:
+                                    cnt += 1
+                                record_counter('daemons.conveyor.poller.update_request_state.%s' % ret)
+
+                    # should touch transfers.
+                    # Otherwise if one bulk transfer includes many requests and one is not terminated, the transfer will be poll again.
+                    transfer_core.touch_transfer(external_host, transfer_id)
+                except (DatabaseException, DatabaseError) as error:
+                    if re.match('.*ORA-00054.*', error.args[0]) or re.match('.*ORA-00060.*', error.args[0]) or 'ERROR 1205 (HY000)' in error.args[0]:
+                        logging.warn(prepend_str + "Lock detected when handling request %s - skipping" % request_id)
+                    else:
+                        logging.error(traceback.format_exc())
+            logging.debug(prepend_str + 'Finished updating %s transfer requests status (%i requests state changed) in %s seconds' % (len(xfers), cnt, (time.time() - tss)))
     except Exception:
         logging.error(traceback.format_exc())
