@@ -27,6 +27,7 @@
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Eric Vaandering <ewv@fnal.gov>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -45,10 +46,11 @@ from traceback import format_exception
 
 
 from rucio.db.sqla.constants import DIDType, SubscriptionState
-from rucio.common.exception import (DatabaseException, DataIdentifierNotFound, InvalidReplicationRule, DuplicateRule, RSEBlacklisted,
-                                    InvalidRSEExpression, InsufficientTargetRSEs, InsufficientAccountLimit, InputValidationError, RSEOverQuota,
-                                    ReplicationRuleCreationTemporaryFailed, InvalidRuleWeight, StagingAreaRuleRequiresLifetime,
-                                    SubscriptionWrongParameter, SubscriptionNotFound)
+from rucio.common.exception import (DatabaseException, DataIdentifierNotFound, InvalidReplicationRule, DuplicateRule,
+                                    RSEBlacklisted, RSEWriteBlocked, InvalidRSEExpression, InsufficientTargetRSEs,
+                                    InsufficientAccountLimit, InputValidationError, RSEOverQuota,
+                                    ReplicationRuleCreationTemporaryFailed, InvalidRuleWeight,
+                                    StagingAreaRuleRequiresLifetime, SubscriptionWrongParameter, SubscriptionNotFound)
 from rucio.common.config import config_get
 from rucio.common.schema import validate_schema
 from rucio.common.utils import chunks
@@ -302,6 +304,8 @@ def transmogrifier(bulk=5, once=False, sleep_time=60):
                                         purge_replicas = False
                                     rse_expression = str(rule_dict['rse_expression'])
                                     comment = str(subscription['comments'])
+                                    if 'comments' in rule_dict:
+                                        comment = str(rule_dict['comments'])
                                     subscription_id = str(subscription['id'])
                                     account = subscription['account']
                                     copies = int(rule_dict['copies'])
@@ -404,7 +408,9 @@ def transmogrifier(bulk=5, once=False, sleep_time=60):
                                             logging.error(prepend_str + '%s' % (str(error)))
                                             monitor.record_counter(counters='transmogrifier.addnewrule.errortype.%s' % (str(error.__class__.__name__)), delta=1)
                                             break
-                                        except (ReplicationRuleCreationTemporaryFailed, InsufficientTargetRSEs, InsufficientAccountLimit, DatabaseException, RSEBlacklisted) as error:
+                                        except (ReplicationRuleCreationTemporaryFailed, InsufficientTargetRSEs,
+                                                InsufficientAccountLimit, DatabaseException, RSEBlacklisted,
+                                                RSEWriteBlocked) as error:
                                             # Errors to be retried
                                             logging.error(prepend_str + '%s Will perform an other attempt %i/%i' % (str(error), attempt + 1, nattempt))
                                             monitor.record_counter(counters='transmogrifier.addnewrule.errortype.%s' % (str(error.__class__.__name__)), delta=1)
