@@ -1,4 +1,4 @@
-# Copyright 2015-2019 CERN for the benefit of the ATLAS collaboration.
+# Copyright 2015-2020 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,17 +14,21 @@
 #
 # Authors:
 # - Vincent Garonne, <vincent.garonne@cern.ch>, 2012-2017
-# - Mario Lassnig, <mario.lassnig@cern.ch>, 2012-2019
+# - Mario Lassnig, <mario.lassnig@cern.ch>, 2012-2020
 # - Angelos Molfetas, <angelos.molfetas@cern.ch>, 2012
 # - Ralph Vigne, <ralph.vigne@cern.ch>, 2013
-# - Cedric Serfon, <cedric.serfon@cern.ch>, 2013-2018
+# - Cedric Serfon, <cedric.serfon@cern.ch>, 2013-2020
 # - Martin Barisits, <martin.barisits@cern.ch>, 2013-2020
 # - Wen Guan, <wen.guan@cern.ch>, 2015
 # - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2015-2019
-# - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2019
+# - asket <asket.agarwal96@gmail.com>, 2018
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Ruturaj Gujar, <ruturaj.gujar23@gmail.com>, 2019
-# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019
+# - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
+# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019-2020
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
+# - James Perry <j.perry@epcc.ed.ac.uk>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -43,7 +47,7 @@ from sqlalchemy.sql import Delete
 from sqlalchemy.types import LargeBinary
 
 from rucio.common import utils
-from rucio.common.schema import NAME_LENGTH, SCOPE_LENGTH
+from rucio.common.schema import get_schema_value
 from rucio.db.sqla.constants import (AccountStatus, AccountType, DIDAvailability, DIDType, DIDReEvaluation,
                                      KeyType, IdentityType, LockState, RuleGrouping, BadFilesStatus,
                                      RuleState, ReplicaState, RequestState, RequestType, RSEType,
@@ -280,7 +284,7 @@ class SoftModelBase(ModelBase):
                                   CheckConstraint('DELETED IS NOT NULL', name=cls.__tablename__.upper() + '_DELETED_NN'),
                                   {'mysql_engine': 'InnoDB', 'info': {'soft_delete': True}})
 
-    def delete(self, session=None):
+    def delete(self, flush=True, session=None):
         """Delete this object"""
         self.deleted = True
         self.deleted_at = datetime.datetime.utcnow()
@@ -290,7 +294,7 @@ class SoftModelBase(ModelBase):
 class Account(BASE, ModelBase):
     """Represents an account"""
     __tablename__ = 'accounts'
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     account_type = Column(AccountType.db_type(name='ACCOUNTS_TYPE_CHK'))
     status = Column(AccountStatus.db_type(default=AccountStatus.ACTIVE, name='ACCOUNTS_STATUS_CHK'))
     email = Column(String(255))
@@ -304,7 +308,7 @@ class Account(BASE, ModelBase):
 class AccountAttrAssociation(BASE, ModelBase):
     """Represents an account"""
     __tablename__ = 'account_attr_map'
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     key = Column(String(255))
     value = Column(BooleanString(255))
     _table_args = (PrimaryKeyConstraint('account', 'key', name='ACCOUNT_ATTR_MAP_PK'),
@@ -331,7 +335,7 @@ class IdentityAccountAssociation(BASE, ModelBase):
     __tablename__ = 'account_map'
     identity = Column(String(2048))
     identity_type = Column(IdentityType.db_type(name='ACCOUNT_MAP_ID_TYPE_CHK'))
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     is_default = Column(Boolean(name='ACCOUNT_MAP_DEFAULT_CHK'), default=False)
     _table_args = (PrimaryKeyConstraint('identity', 'identity_type', 'account', name='ACCOUNT_MAP_PK'),
                    ForeignKeyConstraint(['account'], ['accounts.account'], name='ACCOUNT_MAP_ACCOUNT_FK'),
@@ -343,8 +347,8 @@ class IdentityAccountAssociation(BASE, ModelBase):
 class Scope(BASE, ModelBase):
     """Represents a scope"""
     __tablename__ = 'scopes'
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    account = Column(InternalAccountString(25))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     is_default = Column(Boolean(name='SCOPES_DEFAULT_CHK'), default=False)
     status = Column(ScopeStatus.db_type(name='SCOPE_STATUS_CHK', default=ScopeStatus.OPEN))
     closed_at = Column(DateTime)
@@ -359,9 +363,9 @@ class Scope(BASE, ModelBase):
 class DataIdentifier(BASE, ModelBase):
     """Represents a dataset"""
     __tablename__ = 'dids'
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
-    account = Column(InternalAccountString(25))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     did_type = Column(DIDType.db_type(name='DIDS_TYPE_CHK'))
     is_open = Column(Boolean(name='DIDS_IS_OPEN_CHK'))
     monotonic = Column(Boolean(name='DIDS_MONOTONIC_CHK'), server_default='0')
@@ -416,8 +420,8 @@ class DataIdentifier(BASE, ModelBase):
 
 class DidMeta(BASE, ModelBase):
     __tablename__ = 'did_meta'
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     meta = Column(JSON())
     _table_args = (PrimaryKeyConstraint('scope', 'name', name='DID_META_PK'),
                    ForeignKeyConstraint(['scope', 'name'], ['dids.scope', 'dids.name'], name='DID_META_FK'),)
@@ -426,9 +430,9 @@ class DidMeta(BASE, ModelBase):
 class DeletedDataIdentifier(BASE, ModelBase):
     """Represents a dataset"""
     __tablename__ = 'deleted_dids'
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
-    account = Column(InternalAccountString(25))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     did_type = Column(DIDType.db_type(name='DEL_DIDS_TYPE_CHK'))
     is_open = Column(Boolean(name='DEL_DIDS_IS_OPEN_CHK'))
     monotonic = Column(Boolean(name='DEL_DIDS_MONO_CHK'), server_default='0')
@@ -474,8 +478,8 @@ class UpdatedDID(BASE, ModelBase):
     """Represents the recently updated dids"""
     __tablename__ = 'updated_dids'
     id = Column(GUID(), default=utils.generate_uuid)
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     rule_evaluation_action = Column(DIDReEvaluation.db_type(name='UPDATED_DIDS_RULE_EVAL_ACT_CHK'))
     _table_args = (PrimaryKeyConstraint('id', name='UPDATED_DIDS_PK'),
                    CheckConstraint('SCOPE IS NOT NULL', name='UPDATED_DIDS_SCOPE_NN'),
@@ -486,12 +490,12 @@ class UpdatedDID(BASE, ModelBase):
 class BadReplicas(BASE, ModelBase):
     """Represents the suspicious or bad replicas"""
     __tablename__ = 'bad_replicas'
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     rse_id = Column(GUID())
     reason = Column(String(255))
     state = Column(BadFilesStatus.db_type(name='BAD_REPLICAS_STATE_CHK'), default=BadFilesStatus.SUSPICIOUS)
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     bytes = Column(BigInteger)
     expires_at = Column(DateTime)
     _table_args = (PrimaryKeyConstraint('scope', 'name', 'rse_id', 'state', 'created_at', name='BAD_REPLICAS_STATE_PK'),
@@ -509,7 +513,7 @@ class BadPFNs(BASE, ModelBase):
     path = Column(String(2048))  # PREFIX + PFN
     state = Column(BadPFNStatus.db_type(name='BAD_PFNS_STATE_CHK'), default=BadPFNStatus.SUSPICIOUS)
     reason = Column(String(255))
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     expires_at = Column(DateTime)
     _table_args = (PrimaryKeyConstraint('path', 'state', name='BAD_PFNS_PK'),
                    ForeignKeyConstraint(['account'], ['accounts.account'], name='BAD_PFNS_ACCOUNT_FK'))
@@ -523,8 +527,8 @@ class QuarantinedReplica(BASE, ModelBase, Versioned):
     bytes = Column(BigInteger)
     md5 = Column(String(32))
     adler32 = Column(String(8))
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     _table_args = (PrimaryKeyConstraint('rse_id', 'path', name='QURD_REPLICAS_STATE_PK'),
                    ForeignKeyConstraint(['rse_id'], ['rses.id'], name='QURD_REPLICAS_RSE_ID_FK'),
                    Index('QUARANTINED_REPLICAS_PATH_IDX', 'path', 'rse_id', unique=True))
@@ -555,10 +559,10 @@ class DIDKeyValueAssociation(BASE, ModelBase):
 class DataIdentifierAssociation(BASE, ModelBase):
     """Represents the map between containers/datasets and files"""
     __tablename__ = 'contents'
-    scope = Column(InternalScopeString(SCOPE_LENGTH))  # dataset scope
-    name = Column(String(NAME_LENGTH))    # dataset name
-    child_scope = Column(InternalScopeString(SCOPE_LENGTH))  # Provenance scope
-    child_name = Column(String(NAME_LENGTH))    # Provenance name
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))  # dataset scope
+    name = Column(String(get_schema_value('NAME_LENGTH')))    # dataset name
+    child_scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))  # Provenance scope
+    child_name = Column(String(get_schema_value('NAME_LENGTH')))    # Provenance name
     did_type = Column(DIDType.db_type(name='CONTENTS_DID_TYPE_CHK'))
     child_type = Column(DIDType.db_type(name='CONTENTS_CHILD_TYPE_CHK'))
     bytes = Column(BigInteger)
@@ -578,10 +582,10 @@ class DataIdentifierAssociation(BASE, ModelBase):
 class ConstituentAssociation(BASE, ModelBase):
     """Represents the map between archives and constituents"""
     __tablename__ = 'archive_contents'
-    child_scope = Column(InternalScopeString(SCOPE_LENGTH))    # Constituent file scope
-    child_name = Column(String(NAME_LENGTH))    # Constituent file name
-    scope = Column(InternalScopeString(SCOPE_LENGTH))          # Archive file scope
-    name = Column(String(NAME_LENGTH))          # Archive file name
+    child_scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))    # Constituent file scope
+    child_name = Column(String(get_schema_value('NAME_LENGTH')))    # Constituent file name
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))          # Archive file scope
+    name = Column(String(get_schema_value('NAME_LENGTH')))          # Archive file name
     bytes = Column(BigInteger)
     adler32 = Column(String(8))
     md5 = Column(String(32))
@@ -601,10 +605,10 @@ class ConstituentAssociation(BASE, ModelBase):
 class ConstituentAssociationHistory(BASE, ModelBase):
     """Represents the map between archives and constituents"""
     __tablename__ = 'archive_contents_history'
-    child_scope = Column(InternalScopeString(SCOPE_LENGTH))    # Constituent file scope
-    child_name = Column(String(NAME_LENGTH))    # Constituent file name
-    scope = Column(InternalScopeString(SCOPE_LENGTH))          # Archive file scope
-    name = Column(String(NAME_LENGTH))  # Archive file name
+    child_scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))    # Constituent file scope
+    child_name = Column(String(get_schema_value('NAME_LENGTH')))    # Constituent file name
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))          # Archive file scope
+    name = Column(String(get_schema_value('NAME_LENGTH')))  # Archive file name
     bytes = Column(BigInteger)
     adler32 = Column(String(8))
     md5 = Column(String(32))
@@ -621,10 +625,10 @@ class ConstituentAssociationHistory(BASE, ModelBase):
 class DataIdentifierAssociationHistory(BASE, ModelBase):
     """Represents the map history between containers/datasets and files"""
     __tablename__ = 'contents_history'
-    scope = Column(InternalScopeString(SCOPE_LENGTH))          # dataset scope
-    name = Column(String(NAME_LENGTH))  # dataset name
-    child_scope = Column(InternalScopeString(SCOPE_LENGTH))          # Provenance scope
-    child_name = Column(String(NAME_LENGTH))  # Provenance name
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))          # dataset scope
+    name = Column(String(get_schema_value('NAME_LENGTH')))  # dataset name
+    child_scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))          # Provenance scope
+    child_name = Column(String(get_schema_value('NAME_LENGTH')))  # Provenance name
     did_type = Column(DIDType.db_type(name='CONTENTS_HIST_DID_TYPE_CHK'))
     child_type = Column(DIDType.db_type(name='CONTENTS_HIST_CHILD_TYPE_CHK'))
     bytes = Column(BigInteger)
@@ -665,7 +669,7 @@ class RSE(BASE, SoftModelBase):
     latitude = Column(Float())
     availability = Column(Integer, server_default='7')
     usage = relationship("RSEUsage", order_by="RSEUsage.rse_id", backref="rses")
-#    replicas = relationship("RSEFileAssociation", order_by="RSEFileAssociation.rse_id", backref="rses")
+    qos_class = Column(String(64))
     _table_args = (PrimaryKeyConstraint('id', name='RSES_PK'),
                    UniqueConstraint('rse', 'vo', name='RSES_RSE_UQ'),
                    CheckConstraint('RSE IS NOT NULL', name='RSES_RSE__NN'),
@@ -762,10 +766,19 @@ class RSEProtocols(BASE, ModelBase):
                    CheckConstraint('IMPL IS NOT NULL', name='RSE_PROTOCOLS_IMPL_NN'))
 
 
+class RSEQoSAssociation(BASE, ModelBase):
+    """Represents the mapping of RSEs"""
+    __tablename__ = 'rse_qos_map'
+    rse_id = Column(GUID())
+    qos_policy = Column(String(64))
+    _table_args = (PrimaryKeyConstraint('rse_id', 'qos_policy', name='RSE_QOS_MAP_PK'),
+                   ForeignKeyConstraint(['rse_id'], ['rses.id'], name='RSE_QOS_MAP_RSE_ID_FK'))
+
+
 class AccountLimit(BASE, ModelBase):
     """Represents account limits"""
     __tablename__ = 'account_limits'
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     rse_id = Column(GUID())
     bytes = Column(BigInteger)
     _table_args = (PrimaryKeyConstraint('account', 'rse_id', name='ACCOUNT_LIMITS_PK'),
@@ -776,7 +789,7 @@ class AccountLimit(BASE, ModelBase):
 class AccountGlobalLimit(BASE, ModelBase):
     """Represents account limits"""
     __tablename__ = 'account_glob_limits'
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     rse_expression = Column(String(3000))
     bytes = Column(BigInteger)
     _table_args = (PrimaryKeyConstraint('account', 'rse_expression', name='ACCOUNT_GLOBAL_LIMITS_PK'),
@@ -786,7 +799,7 @@ class AccountGlobalLimit(BASE, ModelBase):
 class AccountUsage(BASE, ModelBase, Versioned):
     """Represents account usage"""
     __tablename__ = 'account_usage'
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     rse_id = Column(GUID())
     files = Column(BigInteger)
     bytes = Column(BigInteger)
@@ -799,8 +812,8 @@ class RSEFileAssociation(BASE, ModelBase):
     """Represents the map between locations and files"""
     __tablename__ = 'replicas'
     rse_id = Column(GUID())
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     bytes = Column(BigInteger)
     md5 = Column(String(32))
     adler32 = Column(String(8))
@@ -817,14 +830,14 @@ class RSEFileAssociation(BASE, ModelBase):
                    CheckConstraint('bytes IS NOT NULL', name='REPLICAS_SIZE_NN'),
                    CheckConstraint('lock_cnt IS NOT NULL', name='REPLICAS_LOCK_CNT_NN'),
                    Index('REPLICAS_TOMBSTONE_IDX', 'tombstone'),
-                   Index('REPLICAS_PATH_IDX', 'path', mysql_length=NAME_LENGTH))
+                   Index('REPLICAS_PATH_IDX', 'path', mysql_length=get_schema_value('NAME_LENGTH')))
 
 
 class CollectionReplica(BASE, ModelBase):
     """Represents replicas for datasets/collections"""
     __tablename__ = 'collection_replicas'
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     did_type = Column(DIDType.db_type(name='COLLECTION_REPLICAS_TYPE_CHK'))
     rse_id = Column(GUID())
     bytes = Column(BigInteger)
@@ -845,8 +858,8 @@ class UpdatedCollectionReplica(BASE, ModelBase):
     """Represents updates to replicas for datasets/collections"""
     __tablename__ = 'updated_col_rep'
     id = Column(GUID(), default=utils.generate_uuid)
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     did_type = Column(DIDType.db_type(name='UPDATED_COL_REP_TYPE_CHK'))
     rse_id = Column(GUID())
     _table_args = (PrimaryKeyConstraint('id', name='UPDATED_COL_REP_PK'),
@@ -859,8 +872,8 @@ class RSEFileAssociationHistory(BASE, ModelBase):
     """Represents a short history of the deleted replicas"""
     __tablename__ = 'replicas_history'
     rse_id = Column(GUID())
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     bytes = Column(BigInteger)
     _table_args = (PrimaryKeyConstraint('rse_id', 'scope', 'name', name='REPLICAS_HIST_PK'),
                    ForeignKeyConstraint(['rse_id'], ['rses.id'], name='REPLICAS_HIST_RSE_ID_FK'),
@@ -873,9 +886,9 @@ class ReplicationRule(BASE, ModelBase):
     __tablename__ = 'rules'
     id = Column(GUID(), default=utils.generate_uuid)
     subscription_id = Column(GUID())
-    account = Column(InternalAccountString(25))
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     did_type = Column(DIDType.db_type(name='RULES_DID_TYPE_CHK'))
     state = Column(RuleState.db_type(name='RULES_STATE_CHK'), default=RuleState.REPLICATING)
     error = Column(String(255))
@@ -930,9 +943,9 @@ class ReplicationRuleHistoryRecent(BASE, ModelBase):
     __tablename__ = 'rules_hist_recent'
     id = Column(GUID())
     subscription_id = Column(GUID())
-    account = Column(InternalAccountString(25))
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     did_type = Column(DIDType.db_type(name='RULES_HIST_RECENT_DIDTYPE_CHK'))
     state = Column(RuleState.db_type(name='RULES_HIST_RECENT_STATE_CHK'))
     error = Column(String(255))
@@ -970,9 +983,9 @@ class ReplicationRuleHistory(BASE, ModelBase):
     __tablename__ = 'rules_history'
     id = Column(GUID())
     subscription_id = Column(GUID())
-    account = Column(InternalAccountString(25))
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     did_type = Column(DIDType.db_type(name='RULES_HISTORY_DIDTYPE_CHK'))
     state = Column(RuleState.db_type(name='RULES_HISTORY_STATE_CHK'))
     error = Column(String(255))
@@ -1007,11 +1020,11 @@ class ReplicationRuleHistory(BASE, ModelBase):
 class ReplicaLock(BASE, ModelBase):
     """Represents replica locks"""
     __tablename__ = 'locks'
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     rule_id = Column(GUID())
     rse_id = Column(GUID())
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     bytes = Column(BigInteger)
     state = Column(LockState.db_type(name='LOCKS_STATE_CHK'), default=LockState.REPLICATING)
     repair_cnt = Column(BigInteger)
@@ -1027,11 +1040,11 @@ class ReplicaLock(BASE, ModelBase):
 class DatasetLock(BASE, ModelBase):
     """Represents dataset locks"""
     __tablename__ = 'dataset_locks'
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     rule_id = Column(GUID())
     rse_id = Column(GUID())
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     state = Column(LockState.db_type(name='DATASET_LOCKS_STATE_CHK'), default=LockState.REPLICATING)
     length = Column(BigInteger)
     bytes = Column(BigInteger)
@@ -1050,7 +1063,7 @@ class UpdatedAccountCounter(BASE, ModelBase):
     """Represents the recently updated Account counters"""
     __tablename__ = 'updated_account_counters'
     id = Column(GUID(), default=utils.generate_uuid)
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     rse_id = Column(GUID())
     files = Column(BigInteger)
     bytes = Column(BigInteger)
@@ -1065,8 +1078,8 @@ class Request(BASE, ModelBase, Versioned):
     __tablename__ = 'requests'
     id = Column(GUID(), default=utils.generate_uuid)
     request_type = Column(RequestType.db_type(name='REQUESTS_TYPE_CHK'), default=RequestType.TRANSFER)
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     did_type = Column(DIDType.db_type(name='REQUESTS_DIDTYPE_CHK'), default=DIDType.FILE)
     dest_rse_id = Column(GUID())
     source_rse_id = Column(GUID())
@@ -1092,7 +1105,7 @@ class Request(BASE, ModelBase, Versioned):
     estimated_transferred_at = Column(DateTime)
     staging_started_at = Column(DateTime)
     staging_finished_at = Column(DateTime)
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     requested_at = Column(DateTime)
     priority = Column(Integer)
     _table_args = (PrimaryKeyConstraint('id', name='REQUESTS_PK'),
@@ -1111,8 +1124,8 @@ class Source(BASE, ModelBase, Versioned):
     """Represents source files for transfers"""
     __tablename__ = 'sources'
     request_id = Column(GUID())
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     rse_id = Column(GUID())
     dest_rse_id = Column(GUID())
     url = Column(String(2048))
@@ -1165,7 +1178,7 @@ class Subscription(BASE, ModelBase, Versioned):
     policyid = Column(SmallInteger, server_default='0')
     state = Column(SubscriptionState.db_type(name='SUBSCRIPTIONS_STATE_CHK', default=SubscriptionState.ACTIVE))
     last_processed = Column(DateTime, default=datetime.datetime.utcnow())
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     lifetime = Column(DateTime)
     comments = Column(String(4000))
     retroactive = Column(Boolean(name='SUBSCRIPTIONS_RETROACTIVE_CHK'), default=False)
@@ -1181,7 +1194,7 @@ class Token(BASE, ModelBase):
     """Represents the authentication tokens and their lifetime"""
     __tablename__ = 'tokens'
     token = Column(String(3072))  # account-identity-appid-uuid -> max length: (+ 30 1 255 1 32 1 32)
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     refresh_token = Column(String(315), default=None)
     refresh = Column(Boolean(name='TOKENS_REFRESH_CHK'), default=False)
     refresh_start = Column(DateTime, default=None)
@@ -1201,7 +1214,7 @@ class Token(BASE, ModelBase):
 class OAuthRequest(BASE, ModelBase):
     """Represents the authentication session parameters of OAuth 2.0 requests"""
     __tablename__ = 'oauth_requests'
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     state = Column(String(50))
     nonce = Column(String(50))
     access_msg = Column(String(2048))
@@ -1222,6 +1235,7 @@ class Message(BASE, ModelBase):
     event_type = Column(String(1024))
     payload = Column(String(4000))
     payload_nolimit = Column(Text)
+    services = Column(String(2048))
     _table_args = (PrimaryKeyConstraint('id', name='MESSAGES_ID_PK'),
                    CheckConstraint('EVENT_TYPE IS NOT NULL', name='MESSAGES_EVENT_TYPE_NN'),
                    CheckConstraint('PAYLOAD IS NOT NULL', name='MESSAGES_PAYLOAD_NN'),)
@@ -1234,6 +1248,7 @@ class MessageHistory(BASE, ModelBase):
     event_type = Column(String(1024))
     payload = Column(String(4000))
     payload_nolimit = Column(Text)
+    services = Column(String(2048))
     __mapper_args__ = {
         'primary_key': [id]  # Fake primary key for SQLA
     }
@@ -1271,7 +1286,7 @@ class Heartbeats(BASE, ModelBase):
 class NamingConvention(BASE, ModelBase):
     """Represents naming conventions for name within a scope"""
     __tablename__ = 'naming_conventions'
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
     regexp = Column(String(255))
     convention_type = Column(KeyType.db_type(name='CVT_TYPE_CHK'))
     _table_args = (PrimaryKeyConstraint('scope', name='NAMING_CONVENTIONS_PK'),
@@ -1281,8 +1296,8 @@ class NamingConvention(BASE, ModelBase):
 class TemporaryDataIdentifier(BASE, ModelBase):
     """Represents a temporary DID (pre-merged files, etc.)"""
     __tablename__ = 'tmp_dids'
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     rse_id = Column(GUID())
     path = Column(String(1024))
     bytes = Column(BigInteger)
@@ -1293,8 +1308,8 @@ class TemporaryDataIdentifier(BASE, ModelBase):
     events = Column(BigInteger)
     task_id = Column(Integer())
     panda_id = Column(Integer())
-    parent_scope = Column(InternalScopeString(SCOPE_LENGTH))
-    parent_name = Column(String(NAME_LENGTH))
+    parent_scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    parent_name = Column(String(get_schema_value('NAME_LENGTH')))
     offset = Column(BigInteger)
     _table_args = (PrimaryKeyConstraint('scope', 'name', name='TMP_DIDS_PK'),
                    Index('TMP_DIDS_EXPIRED_AT_IDX', 'expired_at'))
@@ -1304,10 +1319,10 @@ class LifetimeExceptions(BASE, ModelBase):
     """Represents the exceptions to the lifetime model"""
     __tablename__ = 'lifetime_except'
     id = Column(GUID(), default=utils.generate_uuid)
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
     did_type = Column(DIDType.db_type(name='LIFETIME_EXCEPT_TYPE_CHK'))
-    account = Column(InternalAccountString(25))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     pattern = Column(String(255))
     comments = Column(String(4000))
     state = Column(LifetimeExceptionsState.db_type(name='LIFETIME_EXCEPT_STATE_CHK'))
@@ -1331,9 +1346,9 @@ class VO(BASE, ModelBase):
 class DidsFollowed(BASE, ModelBase):
     """Represents the datasets followed by an user"""
     __tablename__ = 'dids_followed'
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
-    account = Column(InternalAccountString(25))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     did_type = Column(DIDType.db_type(name='DIDS_FOLLOWED_TYPE_CHK'))
     _table_args = (PrimaryKeyConstraint('scope', 'name', 'account', name='DIDS_FOLLOWED_PK'),
                    CheckConstraint('SCOPE IS NOT NULL', name='DIDS_FOLLOWED_SCOPE_NN'),
@@ -1347,9 +1362,9 @@ class DidsFollowed(BASE, ModelBase):
 class FollowEvents(BASE, ModelBase):
     """Represents the events affecting the datasets which are followed"""
     __tablename__ = 'dids_followed_events'
-    scope = Column(InternalScopeString(SCOPE_LENGTH))
-    name = Column(String(NAME_LENGTH))
-    account = Column(InternalAccountString(25))
+    scope = Column(InternalScopeString(get_schema_value('SCOPE_LENGTH')))
+    name = Column(String(get_schema_value('NAME_LENGTH')))
+    account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
     did_type = Column(DIDType.db_type(name='DIDS_FOLLOWED_EVENTS_TYPE_CHK'))
     event_type = Column(String(1024))
     payload = Column(Text)
@@ -1402,6 +1417,7 @@ def register_models(engine):
               RSEFileAssociationHistory,
               RSELimit,
               RSEProtocols,
+              RSEQoSAssociation,
               RSEUsage,
               ReplicaLock,
               ReplicationRule,
@@ -1463,6 +1479,7 @@ def unregister_models(engine):
               RSEFileAssociationHistory,
               RSELimit,
               RSEProtocols,
+              RSEQoSAssociation,
               RSEUsage,
               ReplicaLock,
               ReplicationRule,
