@@ -1,4 +1,4 @@
-# Copyright 2015-2018 CERN for the benefit of the ATLAS collaboration.
+# Copyright 2015-2020 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,11 +13,12 @@
 # limitations under the License.
 #
 # Authors:
-# - Vincent Garonne <vgaronne@gmail.com>, 2015-2018
-# - Wen Guan <wguan.icedew@gmail.com>, 2016
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2015-2018
+# - Wen Guan <wen.guan@cern.ch>, 2016
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2017-2019
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
 # - Eric Vaandering <ewv@fnal.gov>, 2019
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -95,7 +96,7 @@ def mysql_ping_listener(dbapi_conn, connection_rec, connection_proxy):
             raise
 
 
-def mysql_convert_decimal_to_float():
+def mysql_convert_decimal_to_float(pymysql=False):
     """
     The default datatype returned by mysql-python for numerics is decimal.Decimal.
     This type cannot be serialised to JSON, therefore we need to autoconvert to floats.
@@ -106,6 +107,17 @@ def mysql_convert_decimal_to_float():
     """
 
     converter = None
+    if pymysql:
+        try:
+            from pymysql.constants import FIELD_TYPE
+            from pymysql.converters import conversions as conv
+            converter = conv.copy()
+            converter[FIELD_TYPE.DECIMAL] = float
+            converter[FIELD_TYPE.NEWDECIMAL] = float
+        except ImportError:
+            raise RucioException('Trying to use pymysql without having it installed!')
+        return converter
+
     try:
         import MySQLdb.converters  # pylint: disable=import-error
         from MySQLdb.constants import FIELD_TYPE  # pylint: disable=import-error
@@ -167,7 +179,7 @@ def get_engine(echo=True):
                          ('pool_reset_on_return', str), ('use_threadlocal', int)]
         params = {}
         if 'mysql' in sql_connection:
-            conv = mysql_convert_decimal_to_float()
+            conv = mysql_convert_decimal_to_float(pymysql=sql_connection.startswith('mysql+pymysql'))
             params['connect_args'] = {'conv': conv}
         for param, param_type in config_params:
             try:
