@@ -16,7 +16,7 @@
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2012-2014
 # - Vincent Garonne <vgaronne@gmail.com>, 2012-2016
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2019
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2016
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2020
 # - Wen Guan <wguan.icedew@gmail.com>, 2014-2017
 # - Cheng-Hsi Chao <cheng-hsi.chao@cern.ch>, 2014
 # - Tobias Wegner <twegner@cern.ch>, 2017
@@ -26,6 +26,9 @@
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
 # - James Clark <james.clark@physics.gatech.edu>, 2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Eli Chadwick, <eli.chadwick@stfc.ac.uk>, 2020
+# - Luca Scotto Lavina, <scotto@lpnhe.in2p3.fr>, 2021
+# - Thomas Beermann, <thomas.beermann@cern.ch>, 2021
 #
 # PY3K COMPATIBLE
 
@@ -35,6 +38,7 @@ along with some of the default methods for LFN2PFN translations.
 """
 
 import hashlib
+import logging
 
 try:
     # PY2
@@ -55,6 +59,7 @@ if getattr(rsemanager, 'CLIENT_MODE', None):
 if getattr(rsemanager, 'SERVER_MODE', None):
     from rucio.common.types import InternalScope
     from rucio.core import replica
+    from rucio.core.rse import get_rse_vo
 
 
 class RSEDeterministicTranslation(object):
@@ -153,6 +158,28 @@ class RSEDeterministicTranslation(object):
             scope = scope.replace('.', '/')
         return '%s/%s' % (scope, name)
 
+    @staticmethod
+    def __belleii(scope, name, rse, rse_attrs, protocol_attrs):
+        """
+        Given a LFN, convert it directly to a path using the mapping:
+
+            path -> path
+        This is valid only for the belleii convention where the scope can be determined
+        from the LFN using a determinitic function.
+
+        :param scope: Scope of the LFN.
+        :param name: File name of the LFN.
+        :param rse: RSE for PFN (ignored)
+        :param rse_attrs: RSE attributes for PFN (ignored)
+        :param protocol_attrs: RSE protocol attributes for PFN (ignored)
+        :returns: Path for use in the PFN generation.
+        """
+        del scope
+        del rse
+        del rse_attrs
+        del protocol_attrs
+        return name
+
     @staticmethod
     def __ligo(scope, name, rse, rse_attrs, protocol_attrs):
         """
@@ -174,6 +201,24 @@ class RSEDeterministicTranslation(object):
         from ligo_rucio import lfn2pfn as ligo_lfn2pfn  # pylint: disable=import-error
         return ligo_lfn2pfn.ligo_lab(scope, name, None, None, None)
 
+    @staticmethod
+    def __xenon(scope, name, rse, rse_attrs, protocol_attrs):
+        """
+        Given a LFN, turn it into a two level sub-directory structure based on the scope
+        plus a third level based on the name
+        :param scope: Scope of the LFN.
+        :param name: File name of the LFN.
+        :param rse: RSE for PFN (ignored)
+        :param rse_attrs: RSE attributes for PFN (ignored)
+        :param protocol_attrs: RSE protocol attributes for PFN (ignored)
+        :returns: Path for use in the PFN generation.
+        """
+        del rse
+        del rse_attrs
+        del protocol_attrs
+
+        return '%s/%s/%s/%s' % (scope[0:7], scope[4:len(scope)], name.split('-')[0] + "-" + name.split('-')[1], name)
+
     @classmethod
     def _module_init_(cls):
         """
@@ -182,6 +227,7 @@ class RSEDeterministicTranslation(object):
         cls.register(cls.__hash, "hash")
         cls.register(cls.__identity, "identity")
         cls.register(cls.__ligo, "ligo")
+        cls.register(cls.__belleii, "belleii")
         policy_module = None
         try:
             policy_module = config.config_get('policy', 'lfn2pfn_module')
@@ -203,6 +249,9 @@ class RSEDeterministicTranslation(object):
 
             :returns: RSE specific URI of the physical file
         """
+        # ensure that policy package is loaded in case it registers algorithms
+        import rucio.common.schema  # noqa: F401
+
         algorithm = self.rse_attributes.get('lfn2pfn_algorithm', 'default')
         if algorithm == 'default':
             algorithm = RSEDeterministicTranslation._DEFAULT_LFN2PFN
@@ -216,16 +265,19 @@ RSEDeterministicTranslation._module_init_()  # pylint: disable=protected-access
 class RSEProtocol(object):
     """ This class is virtual and acts as a base to inherit new protocols from. It further provides some common functionality which applies for the amjority of the protocols."""
 
-    def __init__(self, protocol_attr, rse_settings):
+    def __init__(self, protocol_attr, rse_settings, logger=None):
         """ Initializes the object with information about the referred RSE.
 
             :param props: Properties of the requested protocol
         """
+        self.auth_token = protocol_attr['auth_token']
+        protocol_attr.pop('auth_token')
         self.attributes = protocol_attr
         self.translator = None
         self.renaming = True
         self.overwrite = False
         self.rse = rse_settings
+        self.logger = logger
         if self.rse['deterministic']:
             self.translator = RSEDeterministicTranslation(self.rse['rse'], rse_settings, self.attributes)
             if getattr(rsemanager, 'CLIENT_MODE', None) and \
@@ -237,6 +289,9 @@ class RSEProtocol(object):
                 setattr(self, 'lfns2pfns', self.__lfns2pfns_client)
             if getattr(rsemanager, 'SERVER_MODE', None):
                 setattr(self, '_get_path', self._get_path_nondeterministic_server)
+        if not self.logger:
+            self.logger = logging.getLogger('%s.null' % __name__)
+            self.logger.disabled = True
 
     def lfns2pfns(self, lfns):
         """
@@ -304,7 +359,8 @@ class RSEProtocol(object):
 
     def _get_path_nondeterministic_server(self, scope, name):  # pylint: disable=invalid-name
         """ Provides the path of a replica for non-deterministic sites. Will be assigned to get path by the __init__ method if neccessary. """
-        scope = InternalScope(scope)
+        vo = get_rse_vo(self.rse['id'])
+        scope = InternalScope(scope, vo=vo)
         rep = replica.get_replica(scope=scope, name=name, rse_id=self.rse['id'])
         if 'path' in rep and rep['path'] is not None:
             path = rep['path']
