@@ -1,5 +1,5 @@
-#!/usr/bin/env python
-# Copyright 2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2012-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,31 +23,28 @@
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-#
-# PY3K COMPATIBLE
+# - Eric Vaandering <ewv@fnal.gov>, 2020
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 
 from __future__ import print_function
+
 from datetime import datetime
 from json import dumps, loads
-from logging import getLogger, StreamHandler, DEBUG
 from traceback import format_exc
-from flask import Flask, Blueprint, Response, request, redirect
+
+from flask import Flask, Blueprint, Response, request, redirect, jsonify
 from flask.views import MethodView
 
 from rucio.api.account import add_account, del_account, get_account_info, list_accounts, list_identities, list_account_attributes, add_account_attribute, del_account_attribute, update_account, get_usage_history
-from rucio.api.identity import add_account_identity, del_account_identity
 from rucio.api.account_limit import get_local_account_limits, get_local_account_limit, get_local_account_usage, get_global_account_limit, get_global_account_limits, get_global_account_usage
+from rucio.api.identity import add_account_identity, del_account_identity
 from rucio.api.rule import list_replication_rules
 from rucio.api.scope import add_scope, get_scopes
 from rucio.common.exception import AccountNotFound, Duplicate, AccessDenied, RucioException, RuleNotFound, RSENotFound, IdentityError, CounterNotFound
-from rucio.common.utils import generate_http_error_flask, APIEncoder, render_json
-from rucio.web.rest.flaskapi.v1.common import before_request, after_request, check_accept_header_wrapper_flask
-
-
-LOGGER = getLogger("rucio.account")
-SH = StreamHandler()
-SH.setLevel(DEBUG)
-LOGGER.addHandler(SH)
+from rucio.common.utils import APIEncoder, render_json
+from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers, check_accept_header_wrapper_flask, try_stream, request_header_ensure_string
+from rucio.web.rest.utils import generate_http_error_flask
 
 
 class Attributes(MethodView):
@@ -75,8 +72,8 @@ class Attributes(MethodView):
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
         except Exception as error:
             print(format_exc())
-            return error, 500
-        return Response(dumps(attribs), content_type="application/json")
+            return str(error), 500
+        return jsonify(attribs)
 
     def post(self, account, key):
         """ Add attributes to an account.
@@ -93,7 +90,7 @@ class Attributes(MethodView):
         :status 404: Account not found.
         :status 500: Database Exception.
         """
-        json_data = request.data
+        json_data = request.data.decode()
         try:
             parameter = loads(json_data)
         except ValueError:
@@ -105,6 +102,7 @@ class Attributes(MethodView):
         except KeyError as error:
             if error.args[0] == 'key' or error.args[0] == 'value':
                 return generate_http_error_flask(400, 'KeyError', '%s not defined' % str(error))
+            raise
         except TypeError:
             return generate_http_error_flask(400, 'TypeError', 'body must be a json dictionary')
 
@@ -117,10 +115,10 @@ class Attributes(MethodView):
         except AccountNotFound as error:
             return generate_http_error_flask(404, 'AccountNotFound', error.args[0])
         except Exception as error:
-            print(str(format_exc()))
-            return error, 500
+            print(format_exc())
+            return str(error), 500
 
-        return "Created", 201
+        return 'Created', 201
 
     def delete(self, account, key):
         """ Remove attribute from account.
@@ -141,9 +139,10 @@ class Attributes(MethodView):
         except AccountNotFound as error:
             return generate_http_error_flask(404, 'AccountNotFound', error.args[0])
         except Exception as error:
-            return error, 500
+            print(format_exc())
+            return str(error), 500
 
-        return "OK", 200
+        return '', 200
 
 
 class Scopes(MethodView):
@@ -171,12 +170,12 @@ class Scopes(MethodView):
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
         except Exception as error:
             print(format_exc())
-            return error, 500
+            return str(error), 500
 
         if not len(scopes):
             return generate_http_error_flask(404, 'ScopeNotFound', 'no scopes found for account ID \'%s\'' % account)
 
-        return Response(dumps(scopes), content_type="application/json")
+        return jsonify(scopes)
 
     def post(self, account, scope):
         """ create scope with given scope name.
@@ -203,9 +202,9 @@ class Scopes(MethodView):
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
         except Exception as error:
             print(format_exc())
-            return error, 500
+            return str(error), 500
 
-        return "Created", 201
+        return 'Created', 201
 
 
 class AccountParameter(MethodView):
@@ -227,7 +226,7 @@ class AccountParameter(MethodView):
         """
         if account == 'whoami':
             # Redirect to the account uri
-            frontend = request.environ.get('HTTP_X_REQUESTED_HOST')
+            frontend = request_header_ensure_string('X-Requested-Host')
             if frontend:
                 return redirect(frontend + "/accounts/%s" % (request.environ.get('issuer')), code=302)
             return redirect(request.environ.get('issuer'), code=303)
@@ -243,17 +242,17 @@ class AccountParameter(MethodView):
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
         except Exception as error:
             print(format_exc())
-            return error, 500
+            return str(error), 500
 
-        dict = acc.to_dict()
+        accdict = acc.to_dict()
 
-        for key, value in dict.items():
+        for key, value in accdict.items():
             if isinstance(value, datetime):
-                dict[key] = value.strftime('%Y-%m-%dT%H:%M:%S')
+                accdict[key] = value.strftime('%Y-%m-%dT%H:%M:%S')
 
-        del dict['_sa_instance_state']
+        del accdict['_sa_instance_state']
 
-        return Response(render_json(**dict), content_type="application/json")
+        return Response(render_json(**accdict), content_type="application/json")
 
     def put(self, account):
         """ update a parameter for a given account name
@@ -267,7 +266,7 @@ class AccountParameter(MethodView):
         :status 404: Account not found.
         :status 500: Database exception.
         """
-        json_data = request.data
+        json_data = request.data.decode()
         try:
             parameter = loads(json_data)
         except ValueError:
@@ -282,9 +281,10 @@ class AccountParameter(MethodView):
             except AccountNotFound as error:
                 return generate_http_error_flask(404, 'AccountNotFound', error.args[0])
             except Exception as error:
-                return error, 500
+                print(format_exc())
+                return str(error), 500
 
-        return "OK", 200
+        return '', 200
 
     def post(self, account):
         """ create account with given account name.
@@ -299,9 +299,8 @@ class AccountParameter(MethodView):
         :status 409: Account already exists.
         :status 500: Database exception.
         """
-        json_data = request.data
         try:
-            parameter = loads(json_data)
+            parameter = loads(request.data.decode())
         except ValueError:
             return generate_http_error_flask(400, 'ValueError', 'cannot decode json parameter dictionary')
 
@@ -311,6 +310,7 @@ class AccountParameter(MethodView):
         except KeyError as error:
             if error.args[0] == 'type':
                 return generate_http_error_flask(400, 'KeyError', '%s not defined' % str(error))
+            raise
         except TypeError:
             return generate_http_error_flask(400, 'TypeError', 'body must be a json dictionary')
         try:
@@ -318,6 +318,7 @@ class AccountParameter(MethodView):
         except KeyError as error:
             if error.args[0] == 'email':
                 return generate_http_error_flask(400, 'KeyError', '%s not defined' % str(error))
+            raise
         except TypeError:
             return generate_http_error_flask(400, 'TypeError', 'body must be a json dictionary')
 
@@ -331,9 +332,9 @@ class AccountParameter(MethodView):
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
         except Exception as error:
             print(format_exc())
-            return error, 500
+            return str(error), 500
 
-        return "Created", 201
+        return 'Created', 201
 
     def delete(self, account):
         """ disable account with given account name.
@@ -353,13 +354,14 @@ class AccountParameter(MethodView):
         except AccountNotFound as error:
             return generate_http_error_flask(404, 'AccountNotFound', error.args[0])
         except Exception as error:
-            return error, 500
+            print(format_exc())
+            return str(error), 500
 
-        return "OK", 200
+        return '', 200
 
 
 class Account(MethodView):
-    @check_accept_header_wrapper_flask(['application/json'])
+    @check_accept_header_wrapper_flask(['application/x-json-stream'])
     def get(self):
         """ list all rucio accounts.
 
@@ -372,16 +374,11 @@ class Account(MethodView):
         :status 500: Database exception
         :returns: A list containing all account names as dict.
         """
+        def generate(_filter, vo):
+            for account in list_accounts(filter=_filter, vo=vo):
+                yield render_json(**account) + "\n"
 
-        filter = {}
-        for k, v in request.args.items():
-            filter[k] = v
-
-        data = ""
-        for account in list_accounts(filter=filter, vo=request.environ.get('vo')):
-            data += render_json(**account) + "\n"
-
-        return Response(data, content_type="application/x-json-stream")
+        return try_stream(generate(_filter=dict(request.args.items(multi=False)), vo=request.environ.get('vo')))
 
 
 class LocalAccountLimits(MethodView):
@@ -459,7 +456,7 @@ class Identities(MethodView):
         :status 404: Account not found.
         :status 500: Database exception.
         """
-        json_data = request.data
+        json_data = request.data.decode()
         try:
             parameter = loads(json_data)
         except ValueError:
@@ -486,13 +483,15 @@ class Identities(MethodView):
             return generate_http_error_flask(409, 'Duplicate', error.args[0])
         except AccountNotFound as error:
             return generate_http_error_flask(404, 'AccountNotFound', error.args[0])
+        except IdentityError as error:
+            return generate_http_error_flask(400, 'IdentityError', error.args[0])
         except Exception as error:
-            print(str(format_exc()))
-            return error, 500
+            print(format_exc())
+            return str(error), 500
 
-        return "Created", 201
+        return 'Created', 201
 
-    @check_accept_header_wrapper_flask(['application/json'])
+    @check_accept_header_wrapper_flask(['application/x-json-stream'])
     def get(self, account):
         """
         Get all identities mapped to an account.
@@ -510,15 +509,16 @@ class Identities(MethodView):
         """
 
         try:
-            data = ""
-            for identity in list_identities(account, vo=request.environ.get('vo')):
-                data += render_json(**identity) + "\n"
-            return Response(data, content_type="application/x-json-stream")
+            def generate(vo):
+                for identity in list_identities(account, vo=vo):
+                    yield render_json(**identity) + "\n"
+
+            return try_stream(generate(request.environ.get('vo')))
         except AccountNotFound as error:
             return generate_http_error_flask(404, 'AccountNotFound', error.args[0])
         except Exception as error:
-            print(str(format_exc()))
-            return error, 500
+            print(format_exc())
+            return str(error), 500
 
     def delete(self, account):
 
@@ -535,7 +535,7 @@ class Identities(MethodView):
         :status 404: Identity not found.
         :status 500: Database exception.
         """
-        json_data = request.data
+        json_data = request.data.decode()
         try:
             parameter = loads(json_data)
         except ValueError:
@@ -558,21 +558,21 @@ class Identities(MethodView):
             return generate_http_error_flask(404, 'IdentityError', error.args[0])
         except Exception as error:
             print(format_exc())
-            return error, 500
+            return str(error), 500
 
-        return "OK", 200
+        return '', 200
 
 
 class Rules(MethodView):
 
-    @check_accept_header_wrapper_flask(['application/json'])
+    @check_accept_header_wrapper_flask(['application/x-json-stream'])
     def get(self, account):
         """
         Return all rules of a given account.
 
         .. :quickref: Rules; Get rules for account.
 
-        :param scope: The scope name.
+        :param account: The account name.
         :resheader Content-Type: application/x-json-stream
         :status 200: OK.
         :status 401: Invalid auth token.
@@ -583,19 +583,19 @@ class Rules(MethodView):
         """
 
         filters = {'account': account}
-        for k, v in request.args.items():
-            filters[k] = v
+        filters.update(request.args)
 
         try:
-            data = ""
-            for rule in list_replication_rules(filters=filters, vo=request.environ.get('vo')):
-                data += dumps(rule, cls=APIEncoder) + '\n'
-            return Response(data, content_type="application/x-json-stream")
+            def generate(vo):
+                for rule in list_replication_rules(filters=filters, vo=vo):
+                    yield dumps(rule, cls=APIEncoder) + '\n'
+
+            return try_stream(generate(vo=request.environ.get('vo')))
         except RuleNotFound as error:
             return generate_http_error_flask(404, 'RuleNotFound', error.args[0])
         except Exception as error:
             print(format_exc())
-            return error, 500
+            return str(error), 500
 
 
 class UsageHistory(MethodView):
@@ -619,7 +619,7 @@ class UsageHistory(MethodView):
         Return the account usage of the account.
         """
         try:
-            return get_usage_history(account=account, rse=None, issuer=request.environ.get('issuer'), vo=request.environ.get('vo'))
+            usage = get_usage_history(account=account, rse=rse, issuer=request.environ.get('issuer'), vo=request.environ.get('vo'))
         except AccountNotFound as error:
             return generate_http_error_flask(404, 'AccountNotFound', error.args[0])
         except CounterNotFound as error:
@@ -628,12 +628,19 @@ class UsageHistory(MethodView):
             return generate_http_error_flask(401, 'AccessDenied', error.args[0])
         except Exception as error:
             print(format_exc())
-            return error, 500
+            return str(error), 500
+
+        for entry in usage:
+            for key, value in entry.items():
+                if isinstance(value, datetime):
+                    entry[key] = value.strftime('%Y-%m-%dT%H:%M:%S')
+
+        return jsonify(usage)
 
 
 class LocalUsage(MethodView):
 
-    @check_accept_header_wrapper_flask(['application/json'])
+    @check_accept_header_wrapper_flask(['application/x-json-stream'])
     def get(self, account, rse=None):
         """
         Return the local account usage of the account.
@@ -652,10 +659,11 @@ class LocalUsage(MethodView):
         """
 
         try:
-            data = ""
-            for usage in get_local_account_usage(account=account, rse=rse, issuer=request.environ.get('issuer'), vo=request.environ.get('vo')):
-                data += dumps(usage, cls=APIEncoder) + '\n'
-            return Response(data, content_type="application/x-json-stream")
+            def generate(issuer, vo):
+                for usage in get_local_account_usage(account=account, rse=rse, issuer=issuer, vo=vo):
+                    yield dumps(usage, cls=APIEncoder) + '\n'
+
+            return try_stream(generate(issuer=request.environ.get('issuer'), vo=request.environ.get('vo')))
         except AccountNotFound as error:
             return generate_http_error_flask(404, 'AccountNotFound', error.args[0])
         except AccessDenied as error:
@@ -664,12 +672,12 @@ class LocalUsage(MethodView):
             return generate_http_error_flask(404, 'RSENotFound', error.args[0])
         except Exception as error:
             print(format_exc())
-            return error, 500
+            return str(error), 500
 
 
 class GlobalUsage(MethodView):
 
-    @check_accept_header_wrapper_flask(['application/json'])
+    @check_accept_header_wrapper_flask(['application/x-json-stream'])
     def get(self, account, rse_expression=None):
         """
         Return the global account usage of the account.
@@ -688,68 +696,68 @@ class GlobalUsage(MethodView):
         """
 
         try:
-            data = ""
-            for usage in get_global_account_usage(account=account, rse_expression=rse_expression, issuer=request.environ.get('issuer'), vo=request.environ.get('vo')):
-                data += dumps(usage, cls=APIEncoder) + '\n'
-            return Response(data, content_type="application/x-json-stream")
+            def generate(vo, issuer):
+                for usage in get_global_account_usage(account=account, rse_expression=rse_expression, issuer=issuer, vo=vo):
+                    yield dumps(usage, cls=APIEncoder) + '\n'
+
+            return try_stream(generate(vo=request.environ.get('vo'), issuer=request.environ.get('issuer')))
         except AccountNotFound as error:
             return generate_http_error_flask(404, 'AccountNotFound', error.args[0])
-        except AccessDenied as error:
-            return generate_http_error_flask(401, 'AccessDenied', error.args[0])
         except RSENotFound as error:
             return generate_http_error_flask(404, 'RSENotFound', error.args[0])
+        except AccessDenied as error:
+            return generate_http_error_flask(401, 'AccessDenied', error.args[0])
         except Exception as error:
             print(format_exc())
-            return error, 500
+            return str(error), 500
 
 
-"""----------------------
-   Web service startup
-----------------------"""
+def blueprint(no_doc=True):
+    bp = Blueprint('account', __name__, url_prefix='/accounts')
 
-bp = Blueprint('account', __name__)
+    attributes_view = Attributes.as_view('attributes')
+    bp.add_url_rule('/<account>/attr/', view_func=attributes_view, methods=['get', ])
+    bp.add_url_rule('/<account>/attr/<key>', view_func=attributes_view, methods=['post', 'delete'])
+    scopes_view = Scopes.as_view('scopes')
+    bp.add_url_rule('/<account>/scopes/', view_func=scopes_view, methods=['get', ])
+    bp.add_url_rule('/<account>/scopes/<scope>', view_func=scopes_view, methods=['post', ])
+    local_account_limits_view = LocalAccountLimits.as_view('local_account_limit')
+    bp.add_url_rule('/<account>/limits/local', view_func=local_account_limits_view, methods=['get', ])
+    bp.add_url_rule('/<account>/limits', view_func=local_account_limits_view, methods=['get', ])
+    bp.add_url_rule('/<account>/limits/local/<rse>', view_func=local_account_limits_view, methods=['get', ])
+    bp.add_url_rule('/<account>/limits/<rse>', view_func=local_account_limits_view, methods=['get', ])
+    global_account_limits_view = GlobalAccountLimits.as_view('global_account_limit')
+    bp.add_url_rule('/<account>/limits/global', view_func=global_account_limits_view, methods=['get', ])
+    bp.add_url_rule('/<account>/limits/global/<rse_expression>', view_func=global_account_limits_view, methods=['get', ])
+    identities_view = Identities.as_view('identities')
+    bp.add_url_rule('/<account>/identities', view_func=identities_view, methods=['get', 'post', 'delete'])
+    rules_view = Rules.as_view('rules')
+    bp.add_url_rule('/<account>/rules', view_func=rules_view, methods=['get', ])
+    usagehistory_view = UsageHistory.as_view('usagehistory')
+    bp.add_url_rule('/<account>/usage/history/<rse>', view_func=usagehistory_view, methods=['get', ])
+    usage_view = LocalUsage.as_view('usage')
+    bp.add_url_rule('/<account>/usage/local', view_func=usage_view, methods=['get', ])
+    bp.add_url_rule('/<account>/usage', view_func=usage_view, methods=['get', ])
+    bp.add_url_rule('/<account>/usage/local/<rse>', view_func=usage_view, methods=['get', ])
+    bp.add_url_rule('/<account>/usage/<rse>', view_func=usage_view, methods=['get', ])
+    global_usage_view = GlobalUsage.as_view('global_usage')
+    bp.add_url_rule('/<account>/usage/global', view_func=global_usage_view, methods=['get', ])
+    bp.add_url_rule('/<account>/usage/global/<rse_expression>', view_func=global_usage_view, methods=['get', ])
+    account_parameter_view = AccountParameter.as_view('account_parameter')
+    bp.add_url_rule('/<account>', view_func=account_parameter_view, methods=['get', 'put', 'post', 'delete'])
+    account_view = Account.as_view('account')
+    if no_doc:
+        # rule without trailing slash needs to be added before rule with trailing slash
+        bp.add_url_rule('', view_func=account_view, methods=['get', ])
+    bp.add_url_rule('/', view_func=account_view, methods=['get', ])
 
-attributes_view = Attributes.as_view('attributes')
-bp.add_url_rule('/<account>/attr', view_func=attributes_view, methods=['get', ])
-bp.add_url_rule('/<account>/attr/<key>', view_func=attributes_view, methods=['post', 'delete'])
-scopes_view = Scopes.as_view('scopes')
-bp.add_url_rule('/<account>/scopes', view_func=scopes_view, methods=['get', ])
-bp.add_url_rule('/<account>/scopes/<scope>', view_func=scopes_view, methods=['post', ])
-account_parameter_view = AccountParameter.as_view('account_parameter')
-bp.add_url_rule('/<account>', view_func=account_parameter_view, methods=['get', 'put', 'post', 'delete'])
-account_view = Account.as_view('account')
-bp.add_url_rule('/', view_func=account_view, methods=['get', ])
-local_account_limits_view = LocalAccountLimits.as_view('local_account_limit')
-bp.add_url_rule('/<account>/limits/local', view_func=local_account_limits_view, methods=['get', ])
-bp.add_url_rule('/<account>/limits/local/<rse>', view_func=local_account_limits_view, methods=['get', ])
-global_account_limits_view = GlobalAccountLimits.as_view('global_account_limit')
-bp.add_url_rule('/<account>/limits/global', view_func=global_account_limits_view, methods=['get', ])
-bp.add_url_rule('/<account>/limits/global/<rse_expression>', view_func=global_account_limits_view, methods=['get', ])
-identities_view = Identities.as_view('identities')
-bp.add_url_rule('/<account>/identities', view_func=identities_view, methods=['get', 'post', 'delete'])
-rules_view = Rules.as_view('rules')
-bp.add_url_rule('/<account>/rules', view_func=rules_view, methods=['get', ])
-usagehistory_view = UsageHistory.as_view('usagehistory')
-bp.add_url_rule('/<account>/usage/history/<rse>', view_func=usagehistory_view, methods=['get', ])
-usage_view = LocalUsage.as_view('usage')
-bp.add_url_rule('/<account>/usage', view_func=usage_view, methods=['get', ])
-bp.add_url_rule('/<account>/usage/<rse>', view_func=usage_view, methods=['get', ])
-global_usage_view = GlobalUsage.as_view('global_usage')
-bp.add_url_rule('/<account>/usage', view_func=global_usage_view, methods=['get', ])
-bp.add_url_rule('/<account>/usage/<rse_expression>', view_func=global_usage_view, methods=['get', ])
-
-application = Flask(__name__)
-application.register_blueprint(bp)
-application.before_request(before_request)
-application.after_request(after_request)
+    bp.before_request(request_auth_env)
+    bp.after_request(response_headers)
+    return bp
 
 
 def make_doc():
-    """ Only used for sphinx documentation to add the prefix """
+    """ Only used for sphinx documentation """
     doc_app = Flask(__name__)
-    doc_app.register_blueprint(bp, url_prefix='/accounts')
+    doc_app.register_blueprint(blueprint(no_doc=False))
     return doc_app
-
-
-if __name__ == "__main__":
-    application.run()
