@@ -1,5 +1,5 @@
-#!/usr/bin/env python
-# Copyright 2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2017-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,76 +15,70 @@
 #
 # Authors:
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2017
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2018
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2018
-# - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2018-2019
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2018
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-#
-# PY3K COMPATIBLE
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 
 from __future__ import print_function
+
 from json import dumps
-from logging import getLogger, StreamHandler, DEBUG
 from traceback import format_exc
 
-from flask import Flask, Blueprint, Response, request
+from flask import Flask, Blueprint, request
 from flask.views import MethodView
 
 from rucio.api.did import list_archive_content
-from rucio.web.rest.flaskapi.v1.common import before_request, after_request, check_accept_header_wrapper_flask
-
-LOGGER, SH = getLogger("rucio.meta"), StreamHandler()
-SH.setLevel(DEBUG)
-LOGGER.addHandler(SH)
+from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers, check_accept_header_wrapper_flask, parse_scope_name, try_stream
+from rucio.web.rest.utils import generate_http_error_flask
 
 
 class Archive(MethodView):
     """ REST APIs for archive. """
 
     @check_accept_header_wrapper_flask(['application/x-json-stream'])
-    def get(self, scope, name):
+    def get(self, scope_name):
         """
         List archive content keys.
 
         .. :quickref: Archive; list archive content keys.
 
-        :param scope: data identifier scope.
-        :param name: data identifier name.
+        :param scope_name: data identifier (scope)/(name).
         :resheader Content-Type: application/x-json-stream
         :status 200: OK.
+        :status 400: Invalid value.
         :status 406: Not Acceptable.
         :status 500: Internal Error.
         """
         try:
-            data = ""
-            for file in list_archive_content(scope=scope, name=name, vo=request.environ.get('vo')):
-                data += dumps(file) + '\n'
-            return Response(data, content_type="application/x-json-stream")
+            scope, name = parse_scope_name(scope_name)
+
+            def generate(vo):
+                for file in list_archive_content(scope=scope, name=name, vo=vo):
+                    yield dumps(file) + '\n'
+
+            return try_stream(generate(vo=request.environ.get('vo')))
+        except ValueError as error:
+            return generate_http_error_flask(400, 'ValueError', error.args[0])
         except Exception as error:
             print(format_exc())
-            return error, 500
+            return str(error), 500
 
 
-"""----------------------
-   Web service startup
-----------------------"""
+def blueprint():
+    bp = Blueprint('archive', __name__, url_prefix='/archives')
 
-bp = Blueprint('archive', __name__)
-archive_view = Archive.as_view('archive')
-bp.add_url_rule('/<scope>/<name>', view_func=archive_view, methods=['get', ])
+    archive_view = Archive.as_view('archive')
+    bp.add_url_rule('/<path:scope_name>/files', view_func=archive_view, methods=['get', ])
 
-application = Flask(__name__)
-application.register_blueprint(bp)
-application.before_request(before_request)
-application.after_request(after_request)
+    bp.before_request(request_auth_env)
+    bp.after_request(response_headers)
+    return bp
 
 
 def make_doc():
-    """ Only used for sphinx documentation to add the prefix """
+    """ Only used for sphinx documentation """
     doc_app = Flask(__name__)
-    doc_app.register_blueprint(bp, url_prefix='/archives')
+    doc_app.register_blueprint(blueprint())
     return doc_app
-
-
-if __name__ == "__main__":
-    application.run()
