@@ -1,5 +1,5 @@
-#!/usr/bin/env python
-# Copyright 2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2014-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,25 +17,21 @@
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2018
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2017
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-#
-# PY3K COMPATIBLE
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 
 import json
-from logging import getLogger, StreamHandler, DEBUG
+from traceback import format_exc
+
 from flask import Flask, Blueprint, Response, request
 from flask.views import MethodView
 
 from rucio.api.heartbeat import list_heartbeats
+from rucio.common.exception import RucioException
 from rucio.common.utils import APIEncoder
-from rucio.web.rest.flaskapi.v1.common import before_request, after_request, check_accept_header_wrapper_flask
-
-
-LOGGER = getLogger("rucio.heartbeat")
-SH = StreamHandler()
-SH.setLevel(DEBUG)
-LOGGER.addHandler(SH)
+from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers, check_accept_header_wrapper_flask
+from rucio.web.rest.utils import generate_http_error_flask
 
 
 class Heartbeat(MethodView):
@@ -54,31 +50,29 @@ class Heartbeat(MethodView):
         :status 406: Not Acceptable.
         :returns: List of heartbeats.
         """
-
-        return Response(json.dumps(list_heartbeats(issuer=request.environ.get('issuer'), vo=request.environ.get('vo')),
-                                   cls=APIEncoder, content_type="application/json"))
+        try:
+            return Response(json.dumps(list_heartbeats(issuer=request.environ.get('issuer'), vo=request.environ.get('vo')),
+                                       cls=APIEncoder), content_type='application/json')
+        except RucioException as error:
+            return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
+        except Exception as error:
+            print(format_exc())
+            return str(error), 500
 
 
-"""----------------------
-   Web service startup
-----------------------"""
-bp = Blueprint('heartbeat', __name__)
+def blueprint():
+    bp = Blueprint('heartbeat', __name__, url_prefix='/heartbeats')
 
-heartbeat_view = Heartbeat.as_view('heartbeat')
-bp.add_url_rule('/', view_func=heartbeat_view, methods=['get', ])
+    heartbeat_view = Heartbeat.as_view('heartbeat')
+    bp.add_url_rule('', view_func=heartbeat_view, methods=['get', ])
 
-application = Flask(__name__)
-application.register_blueprint(bp)
-application.before_request(before_request)
-application.after_request(after_request)
+    bp.before_request(request_auth_env)
+    bp.after_request(response_headers)
+    return bp
 
 
 def make_doc():
-    """ Only used for sphinx documentation to add the prefix """
+    """ Only used for sphinx documentation """
     doc_app = Flask(__name__)
-    doc_app.register_blueprint(bp, url_prefix='/heartbeats')
+    doc_app.register_blueprint(blueprint())
     return doc_app
-
-
-if __name__ == "__main__":
-    application.run()
