@@ -1,5 +1,5 @@
-#!/usr/bin/env python
-# Copyright 2012-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2016-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,23 +15,24 @@
 #
 # Authors:
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2016-2017
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2018
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2018
-# - Thomas Beermann, <thomas.beermann@cern.ch> 2018
-# - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2019
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-#
-# PY3K COMPATIBLE
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 
 from json import loads, dumps
+from traceback import format_exc
 
 from flask import Flask, Blueprint, Response, request
 from flask.views import MethodView
 
 from rucio.api.lifetime_exception import list_exceptions, add_exception, update_exception
 from rucio.common.exception import LifetimeExceptionNotFound, UnsupportedOperation, InvalidObject, RucioException, AccessDenied, LifetimeExceptionDuplicate
-from rucio.common.utils import generate_http_error_flask, APIEncoder
-from rucio.web.rest.flaskapi.v1.common import before_request, after_request, check_accept_header_wrapper_flask
+from rucio.common.utils import APIEncoder
+from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers, check_accept_header_wrapper_flask, try_stream
+from rucio.web.rest.utils import generate_http_error_flask
 
 
 class LifetimeException(MethodView):
@@ -52,16 +53,18 @@ class LifetimeException(MethodView):
         :status 500: Internal Error.
         """
         try:
-            data = ""
-            for exception in list_exceptions(vo=request.environ.get('vo')):
-                data += dumps(exception, cls=APIEncoder) + '\n'
-            return Response(data, content_type="application/x-json-stream")
+            def generate(vo):
+                for exception in list_exceptions(vo=vo):
+                    yield dumps(exception, cls=APIEncoder) + '\n'
+
+            return try_stream(generate(vo=request.environ.get('vo')))
         except LifetimeExceptionNotFound as error:
             return generate_http_error_flask(404, 'LifetimeExceptionNotFound', error.args[0])
         except RucioException as error:
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
         except Exception as error:
-            return error, 500
+            print(format_exc())
+            return str(error), 500
 
     def post(self):
         """
@@ -108,7 +111,8 @@ class LifetimeException(MethodView):
         except RucioException as error:
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
         except Exception as error:
-            return error, 500
+            print(format_exc())
+            return str(error), 500
         return Response(dumps(exception_id), status=201, content_type="application/json")
 
 
@@ -132,17 +136,18 @@ class LifetimeExceptionId(MethodView):
         :returns: List of exceptions.
         """
         try:
-            data = ""
-            for exception in list_exceptions(exception_id, vo=request.environ.get('vo')):
-                data += dumps(exception, cls=APIEncoder) + '\n'
+            def generate(vo):
+                for exception in list_exceptions(exception_id, vo=vo):
+                    yield dumps(exception, cls=APIEncoder) + '\n'
 
-            return Response(data, content_type="application/x-json-stream")
+            return try_stream(generate(vo=request.environ.get('vo')))
         except LifetimeExceptionNotFound as error:
             return generate_http_error_flask(404, 'LifetimeExceptionNotFound', error.args[0])
         except RucioException as error:
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
         except Exception as error:
-            return error, 500
+            print(format_exc())
+            return str(error), 500
 
     def put(self, exception_id):
         """
@@ -178,33 +183,26 @@ class LifetimeExceptionId(MethodView):
         except RucioException as error:
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
         except Exception as error:
-            return error, 500
-        return "Created", 201
-
-# ---------------------
-#   Web service startup
-# ---------------------
+            print(format_exc())
+            return str(error), 500
+        return 'Created', 201
 
 
-bp = Blueprint('lifetime_exception', __name__)
+def blueprint():
+    bp = Blueprint('lifetime_exception', __name__, url_prefix='/lifetime_exceptions')
 
-lifetime_exception_view = LifetimeException.as_view('lifetime_exception')
-bp.add_url_rule('/', view_func=lifetime_exception_view, methods=['get', 'post'])
-lifetime_exception_id_view = LifetimeExceptionId.as_view('lifetime_exception_id')
-bp.add_url_rule('/<exception_id>', view_func=lifetime_exception_id_view, methods=['get', 'put'])
+    lifetime_exception_view = LifetimeException.as_view('lifetime_exception')
+    bp.add_url_rule('/', view_func=lifetime_exception_view, methods=['get', 'post'])
+    lifetime_exception_id_view = LifetimeExceptionId.as_view('lifetime_exception_id')
+    bp.add_url_rule('/<exception_id>', view_func=lifetime_exception_id_view, methods=['get', 'put'])
 
-application = Flask(__name__)
-application.register_blueprint(bp)
-application.before_request(before_request)
-application.after_request(after_request)
+    bp.before_request(request_auth_env)
+    bp.after_request(response_headers)
+    return bp
 
 
 def make_doc():
-    """ Only used for sphinx documentation to add the prefix """
+    """ Only used for sphinx documentation """
     doc_app = Flask(__name__)
-    doc_app.register_blueprint(bp, url_prefix='/lifetime_exceptions')
+    doc_app.register_blueprint(blueprint())
     return doc_app
-
-
-if __name__ == "__main__":
-    application.run()
